/*
  ModBus Slave Test (1)
  
  Based on:
    Object file:    ModBusSlaveTest.spin
    Version:        1.0.0
    Date:           2013
    Author:         Aleksandr Svalov (Alarus on the Forums)
    Email:          alarus@qip.ru
    Licensing:      MIT License - see end of file for terms of use
    Description:    This is a very simple demonstration of the ModBusSlave object.
  
  This example was first run through spin2cpp.exe (v1.5) and then hand updated to 
  make it more approachable. 
  
  For best Conection Daigram viewing, click Tools and and select Font.  Then, select
  Parallax from the font list.
  
  This example assumes use of the Propeller QuickStart's capacitive touch buttons.
   
┌──────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┐
│                                                   Connection Diagram                                                         │
├──────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┤
│                                                                                                                              │
│    ┌───────────┐     ┌─────────────┐                                                                                         │
│    │           │     │             │                                                                                         │
│    │        P31├────┤R1_Out  R1_In├──•                                                                                      │
│    │           │     │             │                                                                                         │
│    │           │     │          GND├──• RS-232 line                                                                          │
│    │           │     │             │                                                                                         │
│    │        P30├────┤T1_In  T1_Out├──•                                                                                      │
│    │           │     │             │                                                                                         │
│    └───────────┘     └─────────────┘                                                                                         │
│      Propeller           Driver                                                                                              │
│         MCU              RS-232                                                                                              │
│      (P8X32A)          (ADM3232E)                                                                                            │
│                                                                                                                              │
│    ┌───────────┐     ┌─────────────┐                                                                                         │
│    │           │     │             │                                                                                         │
│    │        P31├────┤RO          B├──•                                                                                      │
│    │           │  ┌─•RE           │                                                                                         │
│    │        P27├──┫  │             │   RS-485 line                                                                           │
│    │           │  └─┤DE           │                                                                                         │
│    │        P30├────┤DI          A├──•                                                                                      │
│    │           │     │             │                                                                                         │
│    └───────────┘     └─────────────┘                                                                                         │
│      Propeller           Driver                                                                                              │
│         MCU              RS-485                                                                                              │
│      (P8X32A)          (ADM3485E)                                                                                            │
│                                                                                                                              │
└──────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┘
    
*/


#include "ModBusSlave.h"
#include "TouchButtons.h"


#define _Clkmode (1032)
#define _Xinfreq (5000000)
#define Address (1)
#define Quantity (4)
#define Offset (10)
#define Pin_rx (31)
#define Pin_tx (30)
#define Pin_rt (22)
#define Mode (0)
#define Baudrate (115200)


static ModBusSlaveTest bus;


int main() 
{
  bus.Reg_val[0] = 0;
  bus.Reg_val[1] = 100;
  bus.Reg_val[2] = 0;
  bus.Reg_val[3] = 0;
  bus.Reg_min[0] = 0;
  bus.Reg_min[1] = 100;
  bus.Reg_min[2] = 0;
  bus.Reg_min[3] = 0;
  bus.Reg_max[0] = 65535;
  bus.Reg_max[1] = 200;
  bus.Reg_max[2] = 255;
  bus.Reg_max[3] = 0;
  bus.Mbs_reg_address = Address;
  bus.Mbs_reg_quantity = Quantity;
  bus.Mbs_reg_offset = Offset;
  bus.Mbs_reg_mode = Mode;
  bus.Mbs_reg_baudrate = Baudrate;
  bus.Mbs_pin_rx = Pin_rx;
  bus.Mbs_pin_tx = Pin_tx;
  bus.Mbs_pin_rt = Pin_rt;
  bus.Mbs_ptr_registers = (int32_t)(&bus.Reg_val);
  bus.Mbs_ptr_minimum = (int32_t)(&bus.Reg_min);
  bus.Mbs_ptr_maximum = (int32_t)(&bus.Reg_max);
  
  ModBusSlave_Start((int32_t)(&bus.Mbs_reg_address));
  
  TouchButtons_Start((CLKFREQ / 2000));
  
  DIRA = ((DIRA & 0xff00ffff) | 0xff0000);
  
  while (1) 
  {
    OUTA = ((OUTA & 0xff00ffff) | ((bus.Reg_val[2] & 0xff) << 16));
    bus.Reg_min[3] = TouchButtons_State();
    bus.Reg_max[3] = TouchButtons_State();
    bus.Reg_val[3] = TouchButtons_State();
  }
}


/*
┌──────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┐
│                                                   TERMS OF USE: MIT License                                                  │                                                            
├──────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┤
│Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation    │ 
│files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    │
│modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software│
│is furnished to do so, subject to the following conditions:                                                                   │
│                                                                                                                              │
│The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.│
│                                                                                                                              │
│THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          │
│WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         │
│COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   │
│ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         │
└──────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┘
*/